// 6620Params.h: interface for the C6620Params class.
//	A class for managing AD6620 parameters
//////////////////////////////////////////////////////////////////////
// Copyright 2003.    Moe Wheatley AE4JY  <http://www.moetronix.com>
//
//This program is free software; you can redistribute it and/or
//modify it how ever you wish.  Its primary goal is to provide an
// example for 3rd party software developers.  Support will be limited
// depending on my available time.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//
////////////////////////////////////////////////////////////////////////

#if !defined(SDR14_PARAM_6620)
#define SDR14_PARAM_6620

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#pragma pack(push)
#pragma pack(1)

// Defines for all the AD6620 registers
#define ADR_MODECTRL 0x300			//8 bits
 #define MODECTRL_RESET (1<<0)	//pwrs up in soft reset with this bit set
 #define MODECTRL_SREAL (0<<1)
 #define MODECTRL_DREAL (1<<1)
 #define MODECTRL_SCOMPLEX (1<<2)
 #define MODECTRL_SYNCMASTER (1<<3)
#define ADR_NCOCTRL 0x301			//8 bits
 #define NCOCTRL_BYPASS (1<<0)
 #define NCOCTRL_PHZDITHER (1<<1)
 #define NCOCTRL_AMPDITHER (1<<2)
#define ADR_NCOSYNCMASK 0x302	//32 bits (write all 1's)
#define ADR_NCOFREQ 0x303
#define ADR_NCOPHZOFFSET 0x304
#define ADR_CIC2SCALE 0x305	//8 bits, range 0 to 6 each count==6dB atten
#define ADR_CIC2M 0x306		//8 bits, 0 to 15( decimation of 1 to 16 )
#define ADR_CIC5SCALE 0x307	//8 bits, range 0 to 20 each count == 6dB
#define ADR_CIC5M 0x308		//0 to 31(decimation 1 to 32)
#define ADR_RCFCTRL 0x309	// 8 bits
#define ADR_RCFM 0x30A	//8 bits 0 to 31 (decimation 1 to 32)
#define ADR_RCFOFFSET 0x30B	//8 bit filter coef offset for RCF filter
#define ADR_TAPS 0x30C		//8 bits 0 to 255(number of RCF taps 1 to 256)
#define ADR_RESERVED 0x30D	//must be zero


#define MAX_COEF 256

class C6620Params  
{
public:
	void LoadFixedFilter(int groupid);
	INT LoadFromFile(CString sPath);
	C6620Params();
	virtual ~C6620Params();

	BOOL m_NCOphzDither;
	BOOL m_NCOampDither;
	int	m_UseableBW;
	int	m_CIC2Rate;
	int	m_CIC2Scale;
	int	m_CIC5Rate;
	int	m_CIC5Scale;
	int	m_RCFRate;
	int m_RCFTaps;
	int m_FIRcoef[MAX_COEF];
	int m_TotalDecimationRate;

	void LoadCoefficents();

private:
	int GetBufLine( char* inBuf, char* outBuf);
};

#pragma pack(pop)
#endif // !defined(SDR14_PARAM_6620)
